################# TCL FILE #####################################
###############################################################
# Description
# Postprocessor for machine: DMU-35M with HEIDENHAIN TNC-310
# Serial number:
# CNC Control: HAIDENHAIN TNC 310
# Software version: 280-473-XX
#
# Revisions
#
#   Date        Who           Reason
# 27-apr-2000   Satya      Original
# 01-Mar-2002 Oleg Chizh   First Release
# 01-Feb-2003 Kondratenko    
# 12.08.2003  Kondratenko  1.    
#                          2.   'Zero'
#                          3. proc MOM_circular_move
#                          4.  'MOM_path_warning'
#
# $HISTORY$
#
###############################################################
#    %UGII_CAM_POST_DIR%    
#      "ugpost_base.tcl"
	set cam_post_dir [MOM_ask_env_var UGII_CAM_POST_DIR]
#    %UGII_CAM_DEBUG_DIR%    
#    
	set cam_debug_dir [MOM_ask_env_var UGII_CAM_DEBUG_DIR]
#    ugpost_base.tcl      
	source ${cam_post_dir}/ugpost_base.tcl
#       ,     
# source [MOM_ask_env_var UGII_CAM_DEBUG_DIR]mom_debug.tcl ;
# source [MOM_ask_env_var UGII_CAM_DEBUG_DIR]mom_review.tcl ; MOM_set_debug_mode ON

#   G   

set mom_sys_program_stop_code                 		0
set mom_sys_optional_stop_code                		1
set mom_sys_end_of_program_code               		2
set mom_sys_spindle_direction_code(CLW)       		3
set mom_sys_spindle_direction_code(CCLW)      		4
set mom_sys_spindle_direction_code(OFF)       		5
set mom_sys_coolant_code(MIST)                		8
set mom_sys_coolant_code(ON)                  		8
set mom_sys_coolant_code(TAP)                 		8
set mom_sys_coolant_code(FLOOD)               		8
set mom_sys_coolant_code(OFF)                 		9
set mom_sys_rewind_code                       		30

#     
set mom_kin_arc_output_mode                   		FULL_CIRCLE
set mom_kin_arc_valid_planes                  		XYZ
set mom_kin_clamp_time                        		2.0
set mom_kin_flush_time                        		2.0
set mom_kin_linearization_flag                		"F"
set mom_kin_linearization_tol                 		0.001
set mom_kin_machine_resolution                		0.001
set mom_kin_machine_type                      		3_axis_mill
set mom_kin_max_arc_radius                    		29998.999
#set mom_kin_min_arc_length                    "0.005"
set mom_kin_min_arc_radius                    		0.001
set mom_kin_output_unit                       		MM
set mom_kin_post_data_unit                    		MM
set mom_kin_rapid_feed_rate                   		5000
set mom_kin_tool_change_time                  		12.0
set mom_kin_x_axis_limit                      		800
set mom_kin_y_axis_limit                      		800
set mom_kin_z_axis_limit                      		600
set mom_kin_nurbs_output_type				HEIDENHAIN_POLY
set mom_kin_max_fpm                        		7200
set mom_sys_leader(N)                      ""
set mom_sys_leader(X)                      "X"
set mom_sys_leader(Y)                      "Y"
set mom_sys_leader(Z)                      "Z"
set mom_sys_leader(I)                      "X"
set mom_sys_leader(J)                      "Y"
set mom_sys_leader(K)                      "Z"
set mom_sys_leader(A)			   "A"
set mom_sys_leader(B)			   "B"

#      
set mom_sys_list_output                       		ON
set mom_sys_header_output                     		ON
set mom_sys_list_file_rows                    		150
set mom_sys_list_file_columns                 		100
set mom_sys_warning_output                    		OFF
set mom_sys_group_output                      		OFF
set mom_sys_list_file_suffix                  		lpt
set mom_sys_output_file_suffix                		H

#      
set header_check_status					0
set mom_smooth_path					"OFF"
set mom_tolerance_on_off				"OFF"
set mom_small_steps					"OFF"
#
set status_block_create					0
set mom_point1_pos(0) 					-100.0
set mom_point1_pos(1) 					-100.0
set mom_point1_pos(2) 					-50.0
set mom_point2_pos(0) 					100.0
set mom_point2_pos(1) 					100.0
set mom_point2_pos(2) 					0.0
set mom_offset_block					10.0
#
set mom_cycle_cam					"0"
set mom_cycle_orient					"0"
set mom_cycle_step3					0.2
set mom_cycle_delay					"0.1"
set mom_prev_pos(2)					999999.999
set mom_from_pos(0)					0.0
set mom_from_pos(1)					0.0
set mom_from_pos(2)					100.0
#
set length 						0
set accom_length					0
set max_length						5950
set mom_divide_by					"NONE" ; # "NONE" "LENGTH" "TIME"
set accumulated_time					0
set mom_retract_to					"OFFSET"
set mom_offset_dist					100.0
set d_time						0
set flag_divide						0
#
set cutcom_on 						0
set first_cycle_move					0
set status_tool_change					0
set first_motion_flag					0
set spindle_off_flag 					0
set first_tool						0
#
set mom_tool_corner1_radius                             0
set mom_spindle_speed					1000.0

#=============================================================
proc MOM_start_of_program { } {
#      
#=============================================================
  	global mom_logname mom_date
  	global mom_coolant_status mom_cutcom_status
  	global mom_clamp_status mom_cycle_status
  	global mom_spindle_status mom_cutcom_plane
  	global mom_cutcom_adjust_register mom_tool_adjust_register
  	global mom_tool_length_adjust_register mom_length_comp_register
  	global mom_flush_register mom_wire_cutcom_adjust_register
  	global pb_start_of_program_flag  is_from

  	set is_from ""
    	set pb_start_of_program_flag 0

	#---- Unsetting default setting for variables

  	set mom_coolant_status UNDEFINED
	set mom_cutcom_status  UNDEFINED
  	set mom_clamp_status   UNDEFINED
  	set mom_cycle_status   UNDEFINED
  	set mom_spindle_status UNDEFINED
  	set mom_cutcom_plane   UNDEFINED

  	catch {unset mom_cutcom_adjust_register}
  	catch {unset mom_tool_adjust_register}
  	catch {unset mom_tool_length_adjust_register}
  	catch {unset mom_length_comp_register}
  	catch {unset mom_flush_register}
  	catch {unset mom_wire_cutcom_adjust_register}

	PB_file_name

	OPEN_files ; #   ( )  
  	LIST_FILE_HEADER ; #    ""

}

#=============================================================
proc PB_file_name { } {
#=============================================================
        #      , 
        #      
  	#
  	global mom_output_file_basename mom_output_file_suffix

        ;#  -      (  UGS     )
        ;#        UGPOST -       
        #if {[hiset mom_output_file_suffix]} {
        # if {[string length $mom_output_file_suffix] > 0} {
        #    set mom_output_file_suffix [string toupper $mom_output_file_suffix]
        #    }
        # }

  	#set is_name_ok [regexp -nocase -- {[a-z]} $mom_output_file_basename]
  	#if {$is_name_ok == "1"} {
  	#	MOM_abort "\n\n\t: (  Heidenhain 'TNC-310',  DMU-35M)\n\t   -       *.H\n\n"
  	#}

  	; #   
  	  global prog_number ; #  
     	set var0 ""
     	set fl1 [regexp -- {([0-9])+} $mom_output_file_basename var0]
     	if {$fl1==0} {
 	       MOM_abort "\n\n\t 'TNC-310',  DMU-35M:\n\t :   -   - ,   1  (max 8)\n\n"
     	}
     	#set var1 ""
     	#regsub -nocase -- {(0)*} $var0 "" var1 ;
     	#if {$var1==""} {
 	#	MOM_abort "\n\n\t 'TNC-310',  DMU-35M:\n\t :   -   - ,   1  (max 8),    \n\n"
     	#}
     	set prog_number [format "%s" $var0]

}


#=============================================================
proc PB_start { } {
# MOM_start_of_group -      UG   ,    
#=============================================================
	global mom_output_file_basename mom_group_name mom_operation_name
	global header_check_status

  global prog_name
  set prog_name ""
  if {[info exists mom_operation_name]} { set prog_name $mom_operation_name }
  if {[info exists mom_group_name]} { set prog_name $mom_group_name }

	global prog_number ; #  

	if { $header_check_status == 0} {
		MOM_output_literal "BEGIN PGM $prog_number MM"
		set header_check_status 1

    if [llength [info commands PB_CMD_tool_time_use_init] ] {  PB_CMD_tool_time_use_init  }

	}

}


#=============================================================
proc MOM_machine_mode {} {
#=============================================================
	PB_start
}


#=============================================================
proc  MOM_tool_change {} {
#      
#=============================================================
	global mom_tool_name status_tool_change
	set status_tool_change 0
}


#=============================================================
proc  MOM_first_tool {} {
#       
#=============================================================
	global mom_tool_name
}


#=============================================================
proc  MOM_load_tool {} {
#=============================================================
#       LOAD/TOOL.  
#       MOM_msys.
#=============================================================
	MOM_tool_change
}


#=============================================================
proc MOM_start_of_path { } {
#       
#=============================================================
  global circleNumber ; set circleNumber 0; #     -  (  )
  MOM_force once M_spindle X Y Z R F
}


#=============================================================
proc  MOM_before_motion { } {
#=============================================================
	global mom_motion_event mom_motion_type

	FEEDRATE_SET

	switch $mom_motion_type {
   	ENGAGE { PB_engage_move }
   	APPROACH { PB_approach_move }
   	FIRSTCUT { PB_first_cut }
  }

  if {[llength [info commands PB_CMD_before_motion ] ]} { PB_CMD_before_motion }

}

#=============================================================
proc PB_CMD_before_motion { } {
#=============================================================

	global mom_motion_event mom_motion_type
	global mom_output_file_directory mom_output_file_basename
	global mom_cutting_time
	global mom_seqnum
	global mom_pos  mom_prev_pos
	global ptp_file_name output_extn
	global mom_divide_by mom_live_time
	global mom_retract_to mom_retract_point_pos mom_offset_dist
	global length accom_length max_length
	global total_cutting_time accumulated_time
	global header_check_status status_block_create status_tool_change spindle_off_flag

	set accom_length $mom_seqnum

	set total_cutting_time [expr $mom_cutting_time - $accumulated_time ]

	switch [string toupper $mom_divide_by ] {
	"LENGTH" {
	    if { $accom_length >= $max_length } {
	     if { $mom_motion_type == "STEPOVER" || \
	    	   $mom_motion_type == "RETRACT" || \
	    	    $mom_motion_type == "RETURN" || \
	    	     $mom_motion_type == "DEPARTURE" || \
	    	      $mom_motion_type == "RAPID" } {

		global flag_divide ; set flag_divide 1
		global first_tool ; set first_tool 0

		if {$mom_retract_to == "OFFSET"} {
			  MOM_do_template go_offset
		  } elseif {$mom_retract_to == "POINT"} {
		      MOM_force Once X Y Z
			    MOM_do_template go_point
		}
		MOM_force Once M
		MOM_end_of_program
  	MOM_close_output_file $ptp_file_name

  	#set mom_output_file_basename [ makeFileName $mom_output_file_basename ]
  	set mom_output_file_basename [expr $mom_output_file_basename + 1]
		set ptp_file_name "${mom_output_file_directory}${mom_output_file_basename}${output_extn}"
		if {[file exists $ptp_file_name]} { MOM_remove_file $ptp_file_name }
		#
		MOM_open_output_file $ptp_file_name
		set header_check_status 0
		set status_block_create 0
		set status_tool_change  0
		set spindle_off_flag    0
		MOM_reset_sequence  1  1  1
		MOM_force Once X Y Z F M_spindle
		PB_start
		MOM_force once T
		MOM_msys

		if {$mom_retract_to == "OFFSET"} {
			set  mom_prev_pos(2)  [ expr ($mom_prev_pos(2)+$mom_offset_dist)]
			#MOM_initial_move
			MOM_force Once X Y Z F
			MOM_initial_move_prev
			#   Z
			set  mom_prev_pos(2)  [ expr ($mom_prev_pos(2)-$mom_offset_dist)]
 			MOM_initial_move_prev
		 } elseif {$mom_retract_to == "POINT"} {
				   MOM_force Once X Y Z
           MOM_do_template go_point
 				   MOM_initial_move_prev
			  }

		set accom_length 0

		}
	      }

	   }

	  "TIME" { ; }
  	  "NONE" { ; }
  	 default { ; }
	}

}


#=============================================================
proc MOM_nurbs_move { } {
#=============================================================
#       
#=============================================================
  MOM_abort "\n\n!Error!\n\tC DMU-35M   -.\n\n"

}


#=============================================================
proc MOM_from_move { } {
#=============================================================
  	global mom_motion_type
  	global mom_feed_rate mom_feed_rate_per_rev mom_kin_max_fpm
  	COOLANT_SET ; CUTCOM_SET ; SPINDLE_SET ; RAPID_SET ;  MODES_SET
}


#=============================================================
proc MOM_initial_move { } {
#=============================================================
   global mom_motion_type
   global mom_pos mom_prev_pos
   global mom_smooth_path
   global mom_imported_path_status mom_warning_info mom_operation_name

    MOM_force Once X Y Z F

    if {[info exists mom_imported_path_status]==1} {
     if {"YES"==[string toupper "$mom_imported_path_status"]} {
       set mom_warning_info "WARNING: =$mom_operation_name -   ( -   !)"
       MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
       set mom_smooth_path	"ON"
      }
    }

  if {$mom_motion_type == "RAPID" } {
     if {$mom_pos(2) < $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_xy
      }
   } elseif { $mom_motion_type == "CYCLE" } {
  		MOM_rapid_cycle
  	    } else {
  		MOM_linear_move
  	   }
  PB_CMD_save_last_z

}


#=============================================================
proc MOM_initial_move_prev { } {
#=============================================================
  	global mom_motion_type
  	global mom_pos mom_prev_pos

#  MOM_force Once X Y Z F

  	if {$mom_motion_type == "RAPID" } {

	if {$mom_pos(2) < $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_xy_prev
   		MOM_do_template rapid_cycle_z_prev
   	} else {
   		MOM_do_template rapid_cycle_z_prev
   		MOM_do_template rapid_cycle_xy_prev
   	}
  	} elseif { $mom_motion_type == "CYCLE" } {
    			MOM_do_template rapid_cycle_xy_prev
   			MOM_do_template rapid_cycle_z_prev

  	} else {
	        MOM_do_template feedrate_tnc
		MOM_do_template linear_move_prev
  	}
}


#=============================================================
proc MOM_first_move { } {
#=============================================================
  	global mom_feed_rate mom_feed_rate_per_rev mom_motion_type
  	global mom_kin_max_fpm mom_pos mom_prev_pos

  	COOLANT_SET ; CUTCOM_SET ; SPINDLE_SET ; RAPID_SET ;  MODES_SET
	if {[info exists mom_kin_max_fpm] != 0} {
		if {$mom_feed_rate >= $mom_kin_max_fpm}  {MOM_rapid_move ; return
	} else {MOM_linear_move ; return}}

	if {$mom_pos(2) < $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_xy
	}
}


#=============================================================
proc MOM_end_of_path { } {
#=============================================================
}


#=============================================================
proc MOM_end_of_program { } {
#=============================================================
   	global mom_machine_time

	MOM_do_template end_of_path
	MOM_do_template rewind_of_program

global total_cutting_time
set str2 ""
if [llength [info commands outTime] ] {
   set str2 [ outTime $mom_machine_time ]
   ;# set str2 [ outTime $total_cutting_time ]
}

	set line1 [format "; Machine time = %.1f (min) %s " $mom_machine_time $str2 ]
  MOM_output_literal "$line1"

  global prog_number
	MOM_output_literal "END PGM $prog_number MM"

  	LIST_FILE_TRAILER
  	CLOSE_files

}


#=============================================================
proc MOM_set_modes { } {
#=============================================================
  global mom_output_mode
  global mom_arc_mode
  global mom_feed_rate_output_mode
  global mom_parallel_to_axis
  global mom_machine_mode
  global mom_modes_text
  global mom_warning_info

# "ABSOLUTE", "INCREMENTAL"
  global mom_sys_leader
  if {$mom_output_mode == "ABSOLUTE"} {
      MOM_incremental OFF X Z Y
      set mom_sys_leader(X) "X"
      set mom_sys_leader(Y) "Y"
      set mom_sys_leader(Z) "Z"
    } elseif {$mom_output_mode == "INCREMENTAL"} {
      MOM_incremental ON X Z Y
      set mom_sys_leader(X) "IX"
      set mom_sys_leader(Y) "IY"
      set mom_sys_leader(Z) "IZ"
  }

# "LINEAR", "CIRCULAR"

  if {$mom_arc_mode == "LINEAR"} {
  } elseif {$mom_arc_mode == "CIRCULAR"} {
  }

# "OFF", "IPM", "IPR", "MMPM", "MMPR", "INVERSE"

  if {$mom_feed_rate_output_mode == "FPM"} {
  } elseif {$mom_feed_rate_output_mode == "FPR"} {
  } elseif {$mom_feed_rate_output_mode == "INVERSE"} {
  } elseif {$mom_feed_rate_output_mode == "OFF"} {
  }

# "ZAXIS", "WAXIS", "UAXIS"
  switch $mom_parallel_to_axis {
    ZAXIS { ; }
    default  {
       set mom_warning_info "WARNING:     Z (    Z)"
       MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
       set mom_parallel_to_axis  "ZAXIS"
     }
  }

# "MILL", "TURN", "PUNCH", "LASER", "TORCH", "WIRE"
  switch $mom_machine_mode {
    MILL { ; }
    default  {
       set mom_warning_info "WARNING:       (    MILL)"
       MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
       set  mom_machine_mode  "MILL"
     }
  }

}


#=============================================================
proc MOM_msys { } {
#=============================================================
	global mom_spindle_speed
	global status_block_create mom_tolerance_on_off

	if {$status_block_create == "0"} { MOM_block_create }
	if {$mom_tolerance_on_off == "ON"} {
  		MOM_do_template tolerances_1
  		MOM_do_template tolerances_2
  	}
	MOM_spindle_rpm
}


#=============================================================
proc MOM_spindle_rpm { } {
#=============================================================
	global mom_spindle_speed
	global mom_operation_type mom_operation_name
	global mom_tool_name mom_tool_number mom_tool_diameter mom_tool_corner1_radius
	global status_block_create  mom_contact_status
	global spindle_off_flag status_tool_change line_tool

#         
#       TNC
 if {$status_block_create == "0"} {  MOM_block_create  }

 if {$mom_tool_number == 0} {
   MOM_abort "\n\n!Error!:\n\t      $mom_tool_number   $mom_operation_name\n\n"
 }

 if {$status_tool_change == 0} {

   global flag_divide
   global first_tool
   if {$first_tool==1} {
    	MOM_do_template spindle_off
      set spindle_off_flag 0
    }

   set line "" ;# [format "TOOL CALL %d Z S%.0f" $mom_tool_number $mom_spindle_speed ]

   global mom_imported_path_status
   if {[info exists mom_imported_path_status]==1} {
     if {"NO"==[string toupper "$mom_imported_path_status"]} {
        ;#
 	      append line [format " ; Tool: %s"  $mom_tool_name ]
   	   if {[info exists mom_tool_diameter]==1} { append line [format " R=%.2f"  [expr $mom_tool_diameter/2.0] ] ; }
   	   if {[info exists mom_tool_corner1_radius]==1} {
   	   	if {![ EQ_is_zero $mom_tool_corner1_radius ] } { append line [format " r1=%.2f"  $mom_tool_corner1_radius ] ;	}
   	   }
   	   ;#
      }
   }

   #MOM_output_literal "$line"

   if [llength [info commands PB_CMD_tool_time_use_load] ] {  PB_CMD_tool_time_use_load  }

   set first_tool 1
   global mom_tool_text
   set  mom_tool_text $line
   MOM_do_template tool_call_t
   MOM_force once S
   MOM_do_template tool_call_s
   #MOM_do_template spindle_rpm

   if {$spindle_off_flag == 0}	{
	   if {$mom_spindle_speed == 0.0} {
	   	MOM_abort "\n\n!Error!:\n\t      $mom_spindle_speed   $mom_operation_name\n\n"
	   }
     MOM_do_template spindle_rpm
     set spindle_off_flag 1
   }
   set status_tool_change 1

 } else {

    MOM_do_template tool_call_s
    if {$spindle_off_flag == 0}	{
        MOM_force once M_spindle F
    	  MOM_do_template spindle_rpm
   	    set spindle_off_flag 1
	   }

  }

}


#=============================================================
proc MOM_spindle_off { } {
#=============================================================
	global spindle_off_flag
	set spindle_off_flag 0
  MOM_do_template spindle_off
}


#=============================================================
proc MOM_coolant_on { } {
#=============================================================
  	COOLANT_SET
    MOM_do_template coolant_on
}


#=============================================================
proc MOM_coolant_off { } {
#=============================================================
  	COOLANT_SET
    MOM_do_template coolant_off
}


#=============================================================
proc MOM_cutcom_on { } {
#=============================================================
	global mom_cutcom_mode  mom_cutcom_status
	global cutcom_mode cutcom_on

	set cutcom_mode "R"
  if {$mom_cutcom_mode == "OFF"} {
    		set cutcom_mode "0"
  } elseif {$mom_cutcom_mode == "RIGHT"} {
    		set cutcom_mode "R"
  } elseif {$mom_cutcom_mode == "LEFT"} {
    		set cutcom_mode "L"
  } elseif {$mom_cutcom_mode == "ON"} {
    		set cutcom_mode "R"
  }
  if {$mom_cutcom_status == "OFF"} { set cutcom_mode "0" }
  set cutcom_on 1
}


#=============================================================
proc MOM_cutcom_off { } {
#=============================================================
	global cutcom_mode  cutcom_on
	set cutcom_mode "0"
	set cutcom_on 0
	MOM_force once R
}


#=============================================================
proc MOM_linear_move { } {
#=============================================================
	global cutcom_mode  cutcom_on
	global mom_cutcom_mode mom_cutcom_status
	global mom_smooth_path mom_small_steps
	global mom_motion_distance mom_tool_diameter

  if {$mom_small_steps == "ON" && \
	     $mom_cutcom_status == "ON" && \
			  $mom_motion_distance <= $mom_tool_diameter} {
				MOM_do_template feedrate_tnc
				MOM_do_template linear_move_m97
	} elseif {$mom_smooth_path == "ON"} {
		if {$cutcom_on == 1 } {
			MOM_do_template linear_move
#			set cutcom_on 0
		} else {
			set a [ MOM_ask_address_value R ]
			if {$a == "R" ||$a=="L" } {
				MOM_do_template linear_move
			 } else {
				MOM_do_template feedrate_tnc
				MOM_do_template linear_move_m90
			}
		}
	} else  {
		MOM_do_template linear_move
	}

}


#=============================================================
proc MOM_circular_move { } {
#=============================================================
	global mom_arc_direction
	global mom_arc_axis
	global mom_arc_center

	set mom_arc_axis(0) [format %.7f $mom_arc_axis(0)]
	set mom_arc_axis(1) [format %.7f $mom_arc_axis(1)]
	set mom_arc_axis(2) [format %.7f $mom_arc_axis(2)]

    	set xc  $mom_arc_center(0)
    	set yc  $mom_arc_center(1)
    	set zc  $mom_arc_center(2)

  global xc_Prev yc_Prev zc_Prev
  global circleNumber ; incr circleNumber ;
  set var_variant_circle 1 ; # 1 -   .      ;
  			     #   -   .  

  if {$var_variant_circle==1} {
 	;#
   if {$circleNumber!=1} {
   	;##
	if {$mom_arc_axis(2) == "1" || $mom_arc_axis(2) == "-1"} {
     	  set fl 0
     	  MOM_suppress once K ; MOM_force once  I J ;
     	  if {[EQ_is_equal $xc $xc_Prev]} { incr fl }
     	  if {[EQ_is_equal $yc $yc_Prev]} { incr fl }
     	  if {[EQ_is_equal $fl 2]} { MOM_suppress once  I J ; }
	} elseif {$mom_arc_axis(1) == "1" || $mom_arc_axis(1) == "-1"} {
     	  	set fl 0
          	MOM_suppress once J ; MOM_force once  I K ;
     	  	if {[EQ_is_equal $xc $xc_Prev]} { incr fl ; }
     	  	if {[EQ_is_equal $zc $zc_Prev]} { incr fl ; }
     	  	if {[EQ_is_equal $fl 2]} { MOM_suppress once  I K ; }
	} elseif {$mom_arc_axis(0) == "1" || $mom_arc_axis(0) == "-1"} {
          		set fl 0
     	  		MOM_suppress once I ; MOM_force once  J K ;
     	  		if {[EQ_is_equal $yc $yc_Prev]} { incr fl ; }
     	  		if {[EQ_is_equal $zc $zc_Prev]} { incr fl ; }
     	  		if {[EQ_is_equal $fl 2]} { MOM_suppress once  J K ; }
		}
	;##
    } else {
    	;###
	if {$mom_arc_axis(2) == "1" || $mom_arc_axis(2) == "-1"} {
		MOM_force once I J
		MOM_suppress once K
	} elseif {$mom_arc_axis(1) == "1" || $mom_arc_axis(1) == "-1"} {
		 MOM_force once I K
		 MOM_suppress once J
	} elseif {$mom_arc_axis(0) == "1" || $mom_arc_axis(0) == "-1"} {
		 MOM_force once J K
		 MOM_suppress once I
	}
	;###
    }
     ;#
  } else {
  	;####
	if {$mom_arc_axis(2) == "1" || $mom_arc_axis(2) == "-1"} {
		MOM_force once I J
		MOM_suppress once K
	} elseif {$mom_arc_axis(1) == "1" || $mom_arc_axis(1) == "-1"} {
		 MOM_force once I K
		 MOM_suppress once J
	} elseif {$mom_arc_axis(0) == "1" || $mom_arc_axis(0) == "-1"} {
		 MOM_force once J K
		 MOM_suppress once I
	}
	;####
 }

	;#MOM_do_template feedrate_tnc
   	MOM_do_template circle_center

    if {$mom_arc_axis(2) == "1" || $mom_arc_axis(2) == "-1"} {
		MOM_force once X Y
		MOM_suppress once Z
      } elseif {$mom_arc_axis(1) == "1" || $mom_arc_axis(1) == "-1"} {
			MOM_force once X Z
		 	MOM_suppress once Y
	    } elseif {$mom_arc_axis(0) == "1" ||  $mom_arc_axis(0) == "-1"} {
		 		MOM_force once Y Z
		 		MOM_suppress once X
		    }


   if { $mom_arc_direction == "CLW"} {
      		MOM_do_template circle_end_cw
     } elseif { $mom_arc_direction == "CCLW" } {
     		MOM_do_template circle_end_ccw
   	    }

    set xc_Prev  $mom_arc_center(0)
    set yc_Prev  $mom_arc_center(1)
    set zc_Prev  $mom_arc_center(2)

}


#=============================================================
proc MOM_rapid_cycle { } {
#=============================================================
	global mom_pos mom_prev_pos

	if {$mom_pos(2) < $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_xy
   	}
}


#=============================================================
proc MOM_rapid_move { } {
#=============================================================
	MOM_do_template rapid_move
	PB_CMD_save_last_z
}


#=============================================================
proc MOM_cycle_plane_change { } {
#=============================================================
  global cycle_init_flag

   set cycle_init_flag TRUE
   PB_CMD_cycle_plane_change
   set cycle_init_flag FALSE
}
#=============================================================
proc MOM_drill { } {
#=============================================================
  global cycle_name
  global cycle_init_flag first_cycle_move

  set first_cycle_move 0

   set cycle_init_flag TRUE
   set cycle_name DRILL
   CYCLE_SET
}
#=============================================================
proc  MOM_drill_move { } {
#=============================================================
  global cycle_init_flag

   PB_CMD_cycle_init
   MOM_force Once X Y
   MOM_do_template cycle_drill
#  MOM_do_template cycle_parameters
   set cycle_init_flag FALSE
}
#=============================================================
proc MOM_drill_dwell { } {
#=============================================================
   global cycle_name
  global cycle_init_flag first_cycle_move

  set first_cycle_move 0

   set cycle_init_flag TRUE
   set cycle_name DRILL_DWELL
   CYCLE_SET
}
#=============================================================
proc  MOM_drill_dwell_move { } {
#=============================================================
  global cycle_init_flag

   PB_CMD_cycle_init
   MOM_force Once X Y
   MOM_do_template cycle_drill_dwell
#  MOM_do_template cycle_parameters
   set cycle_init_flag FALSE
}
#=============================================================
proc MOM_drill_deep { } {
#=============================================================
  global cycle_name
  global cycle_init_flag

  set first_cycle_move 0

   set cycle_init_flag TRUE
   set cycle_name DRILL_DEEP
   CYCLE_SET
}
#=============================================================
proc  MOM_drill_deep_move { } {
#=============================================================
  global cycle_init_flag

   PB_CMD_cycle_init
   MOM_force Once X Y
   MOM_do_template cycle_drill_deep
#  MOM_do_template cycle_parameters
   set cycle_init_flag FALSE
}
#=============================================================
proc MOM_drill_break_chip { } {
#=============================================================
  global cycle_name
  global cycle_init_flag first_cycle_move

  set first_cycle_move 0

   set cycle_init_flag TRUE
   set cycle_name DRILL_BREAK_CHIP
   CYCLE_SET
}
#=============================================================
proc  MOM_drill_break_chip_move { } {
#=============================================================
  global cycle_init_flag

   PB_CMD_cycle_init
   MOM_force Once X Y
   MOM_do_template cycle_drill_break_chip
#   MOM_do_template cycle_parameters
   set cycle_init_flag FALSE
}
#=============================================================
proc MOM_tap { } {
#=============================================================
  global cycle_name
  global cycle_init_flag first_cycle_move

  set first_cycle_move 0

   set cycle_init_flag TRUE
   set cycle_name TAP
   CYCLE_SET
}
#=============================================================
proc  MOM_tap_move { } {
#=============================================================
  global cycle_init_flag

   PB_CMD_cycle_init
   MOM_force Once X Y
   MOM_do_template cycle_tap
#  MOM_do_template cycle_parameters
   set cycle_init_flag FALSE
}
#=============================================================
proc MOM_bore { } {
#=============================================================
  global cycle_name
  global cycle_init_flag first_cycle_move

  set first_cycle_move 0

   set cycle_init_flag TRUE
   set cycle_name BORE
   CYCLE_SET
}
#=============================================================
proc  MOM_bore_move { } {
#=============================================================
  global cycle_init_flag

   PB_CMD_cycle_init
   MOM_force Once X Y
   MOM_do_template cycle_bore
   MOM_do_template cycle_parameters
   set cycle_init_flag FALSE
}
#=============================================================
proc MOM_bore_drag { } {
#=============================================================
  global cycle_name
  global cycle_init_flag first_cycle_move

  set first_cycle_move 0

   set cycle_init_flag TRUE
   set cycle_name BORE_DRAG
   CYCLE_SET
}
#=============================================================
proc  MOM_bore_drag_move { } {
#=============================================================
  global cycle_init_flag

   PB_CMD_cycle_init
   MOM_force Once X Y
   MOM_do_template cycle_bore_drag
   MOM_do_template cycle_parameters
   set cycle_init_flag FALSE
}
#=============================================================
proc MOM_bore_no_drag { } {
#=============================================================
  global cycle_name
  global cycle_init_flag first_cycle_move

  set first_cycle_move 0

   set cycle_init_flag TRUE
   set cycle_name BORE_NO_DRAG
   CYCLE_SET
}
#=============================================================
proc  MOM_bore_no_drag_move { } {
#=============================================================
  global cycle_init_flag

   PB_CMD_cycle_init
   MOM_force Once X Y
   MOM_do_template cycle_bore_no_drag
   MOM_do_template cycle_parameters
   set cycle_init_flag FALSE
}
#=============================================================
proc MOM_bore_manual { } {
#=============================================================
 global cycle_name
  global cycle_init_flag first_cycle_move

  set first_cycle_move 0

   set cycle_init_flag TRUE
   set cycle_name BORE_MANUAL
   CYCLE_SET
}
#=============================================================
proc  MOM_bore_manual_move { } {
#=============================================================
  global cycle_init_flag

   PB_CMD_cycle_init
   MOM_force Once X Y
   MOM_do_template cycle_bore_manual
   MOM_do_template cycle_parameters
   set cycle_init_flag FALSE
}
#=============================================================
proc MOM_bore_dwell { } {
#=============================================================
  global cycle_name
  global cycle_init_flag first_cycle_move

  set first_cycle_move 0

   set cycle_init_flag TRUE
   set cycle_name BORE_DWELL
   CYCLE_SET
}
#=============================================================
proc  MOM_bore_dwell_move { } {
#=============================================================
  global cycle_init_flag

   PB_CMD_cycle_init
   MOM_force Once X Y
   MOM_do_template cycle_bore_dwell
   MOM_do_template cycle_parameters
   set cycle_init_flag FALSE
}
#=============================================================
proc MOM_bore_back { } {
#=============================================================
  global cycle_name
  global cycle_init_flag first_cycle_move

  set first_cycle_move 0

   set cycle_init_flag TRUE
   set cycle_name BORE_BACK
   CYCLE_SET
}
#=============================================================
proc  MOM_bore_back_move { } {
#=============================================================
  global cycle_init_flag

   PB_CMD_cycle_init
   MOM_force Once X Y
   MOM_do_template cycle_bore_back
   MOM_do_template cycle_parameters
   set cycle_init_flag FALSE
}
#=============================================================
proc MOM_bore_manual_dwell { } {
#=============================================================
  global cycle_name
  global cycle_init_flag first_cycle_move

  set first_cycle_move 0

   set cycle_init_flag TRUE
   set cycle_name BORE_MANUAL_DWELL
   CYCLE_SET
}
#=============================================================
proc  MOM_bore_manual_dwell_move { } {
#=============================================================
  global cycle_init_flag

   PB_CMD_cycle_init
   MOM_force Once X Y
   MOM_do_template cycle_bore_manual_dwell
   MOM_do_template cycle_parameters
   set cycle_init_flag FALSE
}


#=============================================================
proc PB_CMD_cycle_init { } {
#=============================================================
	global cycle_init_flag

	global mom_motion_event
	global mom_tool_diameter

	global cycle_peck_size cycle_type_number   ;# js
	global js_prev_pos			   ;# diy previous Z height
	global js_return_pos			   ;# returnZ incremental from top of hole

	global mom_pos		;# xy and especially Z of this hole
	global mom_prev_pos	;# ditto - previous hole
	global mom_cycle_step1
	global mom_cycle_retract_mode  mom_cycle_retract_to
	global mom_cycle_feed_to
	global mom_cycle_clearance_plane
	global mom_cycle_delay

	global type_cycle first_cycle_move

MOM_force once cycle_number cycle_rapid_to cycle_feed_to cycle_tap_lead cycle_feedrate
MOM_force once cycle_plunge cycle_dwell_top cycle_surface cycle_dwell cycle_pullback

# Heidenhain cycle types
set cycle_type_number 200 	;# drill & peck
set type_cycle "DRILLING ~"
  switch $mom_motion_event {
   "bore_move" {
   	set cycle_type_number 201	;# ream
	set type_cycle "REAMING ~"
   	}
   "bore_drag_move" {
	set cycle_type_number 202	;# bore
	set type_cycle "BORING ~"
  	}
    "tap_move" {
	set cycle_type_number 207	;# rigid tapping
	set type_cycle "TAPPING NEW ~"
	}
	default { ; }
   }

# peck sizes
set cycle_peck_size [expr ($mom_cycle_feed_to*(-1.0))] 	;# single peck size most cycles
if { $mom_motion_event == "drill_deep_move" || $mom_motion_event == "drill_break_chip_move"} {

    if {$mom_cycle_step1 == 0} {
	set cycle_peck_size  $mom_tool_diameter ;# default peck  if not set
    }  else {
	set cycle_peck_size  $mom_cycle_step1	;# real peck
    }
}

if { $mom_pos(2) != $mom_prev_pos(2) }  {
	set cycle_init_flag  "TRUE"
}

if {$cycle_init_flag == "TRUE"} {

  if {![info exists mom_cycle_delay]} {set mom_cycle_delay 0.0}

 MOM_do_template cycle_block		;# cycle def 200/201 etc

 ########################################
 MOM_set_seq_off ; # MOM_suppress Always N
 ########################################

 MOM_do_template cycle_block_1		;# Q200 setup ( engage ) ht
 #######################################
 #  
 set mom_cycle_feed_to [expr abs($mom_cycle_feed_to)]
 #  
 	global mom_alt_pos
 	global mom_cycle_feed_to_pos
   	global depth_bottom
	set depth_bottom  [ expr $mom_cycle_feed_to_pos(2) - $mom_alt_pos(2) ]
 #######################################
 MOM_do_template cycle_block_2		;# Q201 hole depth ( incremental )

 if { $cycle_type_number == 207 }  {	;# only tapping
    	MOM_do_template cycle_block_239	;# Q239 thread lead
    } else {
 	MOM_do_template cycle_block_3	;# Q206 feedrate units/min
    }

 if { $cycle_type_number == 200 }  {	;# only drilling
 	if {$mom_motion_event == "drill_move" || $mom_motion_event == "drill_move_delay"} {
 		set cycle_peck_size [expr abs($cycle_peck_size)]
 	}
    	MOM_do_template cycle_block_4	;# Q202 peck size
    	MOM_do_template cycle_block_5	;# Q210 dwell above hole hardwired to zero
 }

    if {$first_cycle_move == 0} {
	set mom_pos(2) [expr $mom_pos(2) - $mom_cycle_clearance_plane]
	set first_cycle_move 1
     }

 MOM_do_template cycle_block_6		;# Q203 Z of top of hole ( absolute )
 MOM_do_template cycle_block_7		;# Q211 cycle dwell

# if { $mom_cycle_retract_mode == "AUTO" } {
#	set js_return_pos [expr $js_prev_pos - $mom_pos(2)] ;# calc incr retract
	set js_return_pos $mom_cycle_retract_to
	MOM_do_template cycle_block_8	;# Q204 Zreturn after cycle ( incremental )
#	}

 }

  MOM_set_seq_on ; #  MOM_suppress off N

}


#=============================================================
proc PB_CMD_cycle_plane_change { } {
#=============================================================
# plane change happens when drilling operation goes uphill
# ie - when new hole is higher in Z than prev hole
# retract mode AUTO is return to clearance plane using
# Q204 ( which is like G98 ) so this next bit is only for
# case where cycles stay down

global mom_cycle_retract_mode mom_cycle_rapid_to_pos mom_pos

if { $mom_cycle_retract_mode != "AUTO" } {

	MOM_force Once  G_motion 	;# not sure why i need this
	MOM_do_template cycle_rapidtoZ
  }
}


#=============================================================
#proc MOM_sequence_number { } {
#=============================================================
#}


#=============================================================
proc PB_approach_move { } {
#=============================================================
}


#=============================================================
proc PB_engage_move { } {
#=============================================================
}


#=============================================================
proc PB_first_cut { } {
#=============================================================
}


#=============================================================
proc PB_return_move { } {
#=============================================================
}


#=============================================================
proc MOM_gohome_move { } {
#=============================================================
	global mom_pos
	global mom_prev_pos
	global mom_mcs_goto
	global number_rotate_table
	global part_rotate_table
	set mom_pos(0)	$mom_mcs_goto(0)
	set mom_pos(1)	$mom_mcs_goto(1)
	set mom_pos(2)	$mom_mcs_goto(2)
	set mom_pos(3)	0.0
	if {$mom_pos(2) < $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_xy
   	}
}


#=============================================================
proc MOM_delay { } {
#=============================================================
  global mom_sys_delay_param mom_delay_value
  global mom_delay_revs mom_delay_mode delay_time
# post builder provided format for the current mode:
  if {[info exists mom_sys_delay_param(${mom_delay_mode},FORMAT)] != 0} {
    		MOM_set_address_format dwell $mom_sys_delay_param(${mom_delay_mode},FORMAT)
  }

    switch $mom_delay_mode {
      SECONDS {set delay_time $mom_delay_value}
      default {
         set delay_time $mom_delay_revs ;
         global mom_warning_info
         set mom_warning_info "WARNING:      . !"
         MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
      }
    }

    	MOM_do_template delay_1
    	MOM_do_template delay_2
}


#=============================================================
proc MOM_opstop { } {
#=============================================================
    global circleNumber ; set circleNumber 0; #     -  (  )
    MOM_do_template opstop
}


#=============================================================
proc MOM_auxfun { } {
#=============================================================
	MOM_do_template auxfun
}


#=============================================================
proc MOM_prefun { } {
#=============================================================
	MOM_do_template prefun
}


#=============================================================
proc MOM_stop { } {
#=============================================================
    global circleNumber ; set circleNumber 0; #     -  (  )
    MOM_do_template stop
}

#########################################################################################

#=============================================================
proc MOM_small_step_contour { } {
#=============================================================
	global mom_small_steps

}


#=============================================================
proc MOM_smooth_tool_path { } {
#=============================================================
	global mom_smooth_path
}


#=============================================================
proc MOM_tolerance_cycle { } {
#=============================================================
	global mom_tolerance_on_off
	global mom_operation_type
	global mom_inside_outside_tolerances
	global tolerances
	global temp

	MOM_force once Tol

  	if {$mom_operation_type == "Variable-axis Surface Contouring"  	|| \
		$mom_operation_type == "Cavity Milling" 		|| \
		$mom_operation_type == "Fixed-axis Surface Contouring"  || \
		$mom_operation_type == "Planar Milling"  		|| \
		$mom_operation_type == "Sequential Mill Main Operation" || \
		$mom_operation_type == "Zig-Zag Surface" } {
  		set temp [expr $mom_inside_outside_tolerances(0) + $mom_inside_outside_tolerances(1)]
  		set tolerances [expr $temp/2]
#  		if {$mom_tolerance_on_off == "ON"} {
#  			MOM_do_template tolerances_1
#  			MOM_do_template tolerances_2
#  		}
  	}

}


#=============================================================
proc MOM_spindle_orient { } {
#=============================================================
	global mom_angle_orient
	MOM_force once Angle
	MOM_do_template spindle_orient_1
	MOM_do_template spindle_orient_2
}


#=============================================================
proc  MOM_divide { } {
#=============================================================
	global mom_divide_by mom_length_of_program mom_live_time
	global mom_retract_to mom_offset_dist mom_retract_point_pos
	global max_length
#MOM_output_literal "DIVIDE"
#MOM_output_to_listing_device "DIVIDE"
	set max_length [expr double($mom_length_of_program)]
}


#=============================================================
proc MOM_block_create { } {
#    2-  3-   ,  
#      
#=============================================================
	global mom_point1_pos
	global mom_point2_pos
	global header_check_status
	global status_block_create
	global mom_offset_block

	MOM_force once X Y Z
	set mom_point1_pos(0) [expr $mom_point1_pos(0)-$mom_offset_block]
	set mom_point1_pos(1) [expr $mom_point1_pos(1)-$mom_offset_block]
	set mom_point1_pos(2) [expr $mom_point1_pos(2)-$mom_offset_block]
	set mom_point2_pos(0) [expr $mom_point2_pos(0)+$mom_offset_block]
	set mom_point2_pos(1) [expr $mom_point2_pos(1)+$mom_offset_block]
	MOM_do_template start_block
	MOM_do_template end_block
	set status_block_create 1
}


#=============================================================
proc PB_CMD_save_last_z { } {
#=============================================================
# capture previous z height for use in cycles
# can't always use the simple mom_prev_pos in cycles so need
# a dedicated variable
 global mom_pos
 global js_prev_pos			   ;# diy previous Z height

 set js_prev_pos 0.0 ; # irrelevant if not yet set
 if {[hiset mom_pos(2)]} { set js_prev_pos $mom_pos(2) }

}


#=============================================================
proc  MOM_text {} {
#        .
#=============================================================
  global mom_user_defined_text mom_record_fields mom_field_count mom_record_text
  global mom_Instruction
  global mom_sys_control_out mom_sys_control_in
  global mom_pprint mom_pprint_defined
  global mom_operator_message mom_operator_message_defined
  global mom_warning_info

    switch [string toupper $mom_record_fields(0)] {
	"INSERT" {
		MOM_output_literal $mom_record_text
	}
	"ZERO"	{
		global mom_work_coordinate_number
		set mom_work_coordinate_number $mom_record_fields(1)
		MOM_zero
	}
	"MCSROT" { ; }
	"PARTNO" {
		global prog_number
		set mom_record_text [string trim $mom_record_text " "]
		set temp [split $mom_record_text " "]
		set prog_number [lindex $temp 0]
	}
	default	{ MOM_output_literal "$mom_user_defined_text" }
     }
}


#=============================================================
proc MOM_zero {} {
#        ZERO
#=============================================================
	global mom_work_coordinate_number
	MOM_output_literal "CYCL DEF 7.0 DATUM SHIFT"
	MOM_output_literal "CYCL DEF 7.1 #$mom_work_coordinate_number"
#	MOM_output_literal "CYCL DEF 7.2 Y0.0"
#	MOM_output_literal "CYCL DEF 7.3 Z0.0"
	MOM_output_literal "; SET NULL POINT SHIFT"
			MOM_force once  X Y Z
}


#==================================================================
proc MOM_path_warning { } {
#==================================================================
# This procedure is executed when the in CLSF file has Inconsistent axis.
  global mom_path_warning
  if {[info exists mom_path_warning]} {
     if {[string length $mom_path_warning]!=0} {
     	global mom_seqnum mom_warning_info
     	set tmpNum [format "%.0f" $mom_seqnum]
        set mom_warning_info "Error : $mom_path_warning ( =$tmpNum)"
     	MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
     	; # MOM_abort $mom_warning_info
     	}
   }

}